package Test::Unit::Tutorial;

# this is only a container for the Test::Unit tutorial
# to allow viewing tutorial-style documentation on
# unit testing by way of the perldoc utility.

1;
__END__

=head1 NAME

    Test::Unit::Tutorial - Tutorial on unit testing

=head1 SYNOPSIS

    perldoc Test::Unit::Tutorial

=head1 DESCRIPTION

    Here should be extensive documentation on what unit testing is,
    why it is useful, and how to do it with the Test::Unit collection
    of modules.

    Sorry for not implementing this yet.

    Please have a look at the examples in the examples directory
    and read the README file that came with this distribution.

    A short tutorial on how to use the unit testing framework is
    included in the Test::Unit::TestCase documentation.

    Further examples can be found by looking at the self test
    collection, starting in Test::Unit::tests::AllTests.

=head1 AUTHOR

    Christian Lemburg <lemburg@acm.org>

=head1 SEE ALSO

    - The module documentation for all modules in the Test::Unit tree.
    - Refactoring. Improving The Design Of Existing Code. Martin Fowler. 
      Addison-Wesley, 1999.
    - The JUnit (unit testing framework for Java) documentation. 
    - http://www.xProgramming.com/

=cut
